/**
 * Copyright 2016 the WHATAP project authors. All rights reserved.
 * Use of this source code is governed by a license that
 * can be found in the LICENSE file.
 */


var Value = require ('./value');

function SummaryValue() {
    Value.call(this);

}

SummaryValue.prototype = new Value();
SummaryValue.prototype.constructor = SummaryValue;
SummaryValue.prototype.add = function(value) {};
SummaryValue.prototype.addObject = function(other) {};
SummaryValue.prototype.addCount = function() {};
SummaryValue.prototype.longSum = function() {};
SummaryValue.prototype.longMin = function() {};
SummaryValue.prototype.longMax = function() {};
SummaryValue.prototype.longAvg = function() {};
SummaryValue.prototype.doubleSum = function() {};
SummaryValue.prototype.doubleMin = function() {};
SummaryValue.prototype.doubleMax = function() {};
SummaryValue.prototype.doubleAvg = function() {};
SummaryValue.prototype.getCount = function() {};

module.exports = SummaryValue;