/**
 * Copyright 2016 the WHATAP project authors. All rights reserved.
 * Use of this source code is governed by a license that
 * can be found in the LICENSE file.
 */

exports.NodeAgent = require('./lib/core/agent').NodeAgent;
exports.Tracer = require('./lib/trace/trace-context-manager');
