function num(k, defValue){
    var v=process.env[k];
    return v?Number(v):defValue;
};
function bool(k, defValue){
    var v=process.env[k];
    return v?Boolean(v):defValue;
};
function str(k, defValue){
    var v=process.env[k];
    return v?v:defValue;
};

module.exports = {
    num: num,
    bool: bool,
    str: str
}