class ParamDef {
    static get KEEPALIVE() { return 1; }
    static get AGENT_BOOT_ENV() { return 2; }
    static get CONFIGURE_UPDATE() { return 3; }
    static get CONFIGURE_GET() { return 4; }
    static get COMPO_VERSIONS() { return 5; }
    static get THREAD_LIST() { return 7; }
    static get THREAD_DETAIL() { return 8; }
    static get GET_ACTIVE_STACK() { return 9; }
    static get HEAP_HISTO() { return 10; }
    static get LOADED_CLASS_LIST() { return 11; }
    static get GET_ENV() { return 12; }
    static get SYSTEM_GC() { return 13; }
    static get SET_CONFIG() { return 14; }
    static get OPEN_SOCKET_LIST() { return 15; }
    static get LOADED_CLASS_DETAIL() { return 16; }
    static get LOADED_CLASS_REDEFINE() { return 17; }
    static get AGENT_JAR_LIST() { return 18; }
    static get AGENT_JAR_SAVE() { return 19; }
    static get AGENT_JAR_DELETE() { return 20; }
    static get RESET_STRING_SENT_MARK() { return 21; }
    static get AGENT_LOG_LIST() { return 22; }
    static get AGENT_LOG_READ() { return 23; }
    static get THREAD_CONTROL() { return 24; }
    static get GET_ACTIVE_TRANSACTION_LIST() { return 25; }
    static get GET_ACTIVE_TRANSACTION_DETAIL() { return 26; }
    static get GET_TOPOLOGY() { return 27; }
    static get INFRA_NET_STAT() { return 28; }
    static get INFRA_PS_EF() { return 29; }
    static get INFRA_DOCKER_LIST() { return 30; }
    static get INFRA_REMOTE_CMD() { return 33; }
    static get INFRA_PEER_LIST() { return 34; }
    static get INFRA_AGENT_UPDATE() { return 36; }
    static get AGENT_DUMP_LIST() { return 31; }
    static get AGENT_DUMP_READ() { return 32; }
    static get AGENT_STAT() { return 35; }
    static get JVM_THREAD_DUMP() { return 37; }
    static get JVM_HEAP_DUMP() { return 38; }
    static get KUBENETES() { return 39; }
    static get GET_ACTIVE_STATS() { return 40; }
    static get GET_DBCONN_POOL() { return 41; }
    static get MODULE_DEPENDENCY() { return 101; }
    static get DBX_INFO() { return 201; }
    static get DBX_SETTINGS() { return 202; }
    static get DBX_USER_SCRIPT() { return 207; }
    static get DBX_SCRIPT_LIST() { return 208; }
    static get DBX_SCRIPT_SQL() { return 209; }
    static get DBX_KILL_SESSION() { return 210; }
    static get DBX_GET_ALERT() { return 211; }
    static get DBX_SET_ALERT() { return 212; }
    static get DBX_PLAN() { return 213; }
    static get DBX_SESSION_LIST() { return 214; }
    static get DBX_SET_CPUCORES() { return 215; }
    static get DBX_SESSION_STAT() { return 203; }
    static get DBX_SESSION_EVENT() { return 204; }
    static get DBX_TOPSTAT() { return 205; }
    static get DBX_TOPEVENT() { return 206; }
    static get CONFIG_DBC() { return 301; }
    static get METHOD_PERF_STAT() { return 302; }
    static get BLOCKING_DETECT() { return 303; }
    static get CW_INFO() { return 401; }
    static get CW_EVENTS() { return 402; }
    static get CW_CREDENTIAL() { return 403; }
    static get CW_REGIONS() { return 404; }
    static get CW_METRIC_NAMES() { return 405; }
    static get CW_AWS_RESOURCE_IDS() { return 406; }
    static get CW_STATISTICS() { return 407; }
    static get CW_UNITS() { return 408; }
    static get CW_METRICS() { return 409; }
}

module.exports = ParamDef;