/**
 * Copyright 2016 the WHATAP project authors. All rights reserved.
 * Use of this source code is governed by a license that
 * can be found in the LICENSE file.
 */

function LocalContext(ctx, step, option) {
    this.step = step;
    this.context = ctx;
    this.option = option;
    this.service = false;
};

module.exports = LocalContext;