class PacketTypeEnum {
    static get TX_BLANK() { return 0; }
    static get TX_START() { return 1; }
    static get TX_DB_CONN() { return 2; }
    static get TX_DB_FETCH() { return 3; }
    static get TX_SQL() { return 4; }
    static get TX_SQL_START() { return 5; }
    static get TX_SQL_END() { return 6; }

    static get TX_HTTPC() { return 7; }
    static get TX_HTTPC_START() { return 8; }
    static get TX_HTTPC_END() { return 9; }

    static get TX_ERROR() { return 10; }
    static get TX_MSG() { return 11; }
    static get TX_METHOD() { return 12; }

    static get TX_SECURE_MSG() { return 13; }
    static get TX_WEB_SOCKET() { return 16; }

    static get TX_END() { return 255; }

    static get TX_PARAM() { return 30; }
    static get ACTIVE_STACK() { return 40; }
    static get ACTIVE_STATS() { return 41; }
    static get DBCONN_POOL() { return 42; }

    static get EVENT() { return 50; }

    static get RELAY_PACK() { return 244; }
}

module.exports = PacketTypeEnum;