var StringUtil = {
    isEmpty: function isEmpty(s){
        return s === undefined || s === null || s.length == 0;
    },
    isNotEmpty: function isNotEmpty(s){
        return !s && s.length > 0;
    }
}

module.exports = StringUtil;