/**
 * Copyright 2016 the WHATAP project authors. All rights reserved.
 * Use of this source code is governed by a license that
 * can be found in the LICENSE file.
 */

function Value() {
}

Value.prototype.write = function(dout) {};
Value.prototype.read = function(din) {return this;};
Value.prototype.getValueType = function() {};
Value.prototype.toObject = function() {return this;};

module.exports = Value;