/**
 * Copyright 2016 the WHATAP project authors. All rights reserved.
 * Use of this source code is governed by a license that
 * can be found in the LICENSE file.
 */

function ParsedSql(type, sql, param) {
    this.sql = sql;
    this.type = type;
    this.param = param;
}

module.exports = ParsedSql;
