/**
 * Copyright 2016 the WHATAP project authors. All rights reserved.
 * Use of this source code is governed by a license that
 * can be found in the LICENSE file.
 */

module.exports = CounterTask;

function CounterTask() {};
CounterTask.prototype.process = function (p) {};